import os
from dotenv import load_dotenv
from typing import Optional
import logging

# Load environment variables from .env file
load_dotenv()

logger = logging.getLogger(__name__)

class Config:
    """Configuration class for the Hyperliquid trading bot."""
    
    # Hyperliquid API Configuration
    HYPERLIQUID_PRIVATE_KEY: Optional[str] = os.getenv('HYPERLIQUID_PRIVATE_KEY')
    HYPERLIQUID_SECRET_KEY: Optional[str] = os.getenv('HYPERLIQUID_SECRET_KEY')
    HYPERLIQUID_TESTNET: bool = os.getenv('HYPERLIQUID_TESTNET', 'true').lower() == 'true'
    
    # Trading Bot Configuration
    DEFAULT_TRADING_SYMBOL: str = os.getenv('DEFAULT_TRADING_SYMBOL', 'BTC/USDC:USDC')
    DEFAULT_TRADE_AMOUNT: float = float(os.getenv('DEFAULT_TRADE_AMOUNT', '0.001'))
    RISK_MANAGEMENT_ENABLED: bool = os.getenv('RISK_MANAGEMENT_ENABLED', 'true').lower() == 'true'
    MAX_POSITION_SIZE: float = float(os.getenv('MAX_POSITION_SIZE', '100'))
    STOP_LOSS_PERCENTAGE: float = float(os.getenv('STOP_LOSS_PERCENTAGE', '2.0'))
    TAKE_PROFIT_PERCENTAGE: float = float(os.getenv('TAKE_PROFIT_PERCENTAGE', '5.0'))
    
    # Telegram Bot Configuration
    TELEGRAM_BOT_TOKEN: Optional[str] = os.getenv('TELEGRAM_BOT_TOKEN')
    TELEGRAM_CHAT_ID: Optional[str] = os.getenv('TELEGRAM_CHAT_ID')
    TELEGRAM_ENABLED: bool = os.getenv('TELEGRAM_ENABLED', 'true').lower() == 'true'
    
    # Bot monitoring configuration
    BOT_HEARTBEAT_SECONDS = int(os.getenv('BOT_HEARTBEAT_SECONDS', '30'))
    
    # Logging
    LOG_LEVEL: str = os.getenv('LOG_LEVEL', 'INFO')
    
    # Log file configuration
    LOG_TO_FILE: bool = os.getenv('LOG_TO_FILE', 'true').lower() == 'true'
    LOG_FILE_PATH: str = os.getenv('LOG_FILE_PATH', 'logs/trading_bot.log')
    LOG_MAX_SIZE_MB: int = int(os.getenv('LOG_MAX_SIZE_MB', '10'))
    LOG_BACKUP_COUNT: int = int(os.getenv('LOG_BACKUP_COUNT', '5'))
    LOG_ROTATION_TYPE: str = os.getenv('LOG_ROTATION_TYPE', 'size')  # 'size' or 'time'
    LOG_ROTATION_INTERVAL: str = os.getenv('LOG_ROTATION_INTERVAL', 'midnight')  # for time rotation
    
    @classmethod
    def validate(cls) -> bool:
        """Validate all configuration settings."""
        is_valid = True
        
        # Validate Hyperliquid settings
        if not cls.HYPERLIQUID_PRIVATE_KEY:
            logger.error("❌ HYPERLIQUID_PRIVATE_KEY is required")
            is_valid = False
        
        # Validate Telegram settings (if enabled)
        if cls.TELEGRAM_ENABLED:
            if not cls.TELEGRAM_BOT_TOKEN:
                logger.error("❌ TELEGRAM_BOT_TOKEN is required when Telegram is enabled")
                is_valid = False
            
            if not cls.TELEGRAM_CHAT_ID:
                logger.error("❌ TELEGRAM_CHAT_ID is required when Telegram is enabled")
                is_valid = False
        
        # Validate heartbeat interval
        if cls.BOT_HEARTBEAT_SECONDS < 5:
            logger.error("❌ BOT_HEARTBEAT_SECONDS must be at least 5 seconds to avoid rate limiting")
            is_valid = False
        elif cls.BOT_HEARTBEAT_SECONDS > 600:
            logger.warning("⚠️ BOT_HEARTBEAT_SECONDS is very high (>10 minutes), monitoring may be slow")
        
        # Validate trading settings
        if cls.DEFAULT_TRADE_AMOUNT <= 0:
            logger.error("❌ DEFAULT_TRADE_AMOUNT must be positive")
            is_valid = False
        
        # Validate logging settings
        if cls.LOG_MAX_SIZE_MB <= 0:
            logger.error("❌ LOG_MAX_SIZE_MB must be positive")
            is_valid = False
        
        if cls.LOG_BACKUP_COUNT < 0:
            logger.error("❌ LOG_BACKUP_COUNT must be non-negative")
            is_valid = False
        
        if cls.LOG_ROTATION_TYPE not in ['size', 'time']:
            logger.error("❌ LOG_ROTATION_TYPE must be 'size' or 'time'")
            is_valid = False
        
        return is_valid
    
    @classmethod
    def get_hyperliquid_config(cls) -> dict:
        """Get Hyperliquid configuration in CCXT format."""
        config = {
            'testnet': cls.HYPERLIQUID_TESTNET,
            'sandbox': cls.HYPERLIQUID_TESTNET,  # CCXT uses sandbox for testnet
        }
        
        # Add authentication if available
        if cls.HYPERLIQUID_PRIVATE_KEY:
            config['apiKey'] = cls.HYPERLIQUID_PRIVATE_KEY
            
        if cls.HYPERLIQUID_SECRET_KEY:
            config['secret'] = cls.HYPERLIQUID_SECRET_KEY
            
        # Legacy support for private_key parameter
        if cls.HYPERLIQUID_PRIVATE_KEY:
            config['private_key'] = cls.HYPERLIQUID_PRIVATE_KEY
            
        return config
    
    @classmethod
    def print_config(cls):
        """Print current configuration (hiding sensitive data)."""
        print("🔧 Current Configuration:")
        print(f"  HYPERLIQUID_TESTNET: {cls.HYPERLIQUID_TESTNET}")
        print(f"  DEFAULT_TRADING_SYMBOL: {cls.DEFAULT_TRADING_SYMBOL}")
        print(f"  DEFAULT_TRADE_AMOUNT: {cls.DEFAULT_TRADE_AMOUNT}")
        print(f"  RISK_MANAGEMENT_ENABLED: {cls.RISK_MANAGEMENT_ENABLED}")
        print(f"  MAX_POSITION_SIZE: {cls.MAX_POSITION_SIZE}")
        print(f"  STOP_LOSS_PERCENTAGE: {cls.STOP_LOSS_PERCENTAGE}%")
        print(f"  TAKE_PROFIT_PERCENTAGE: {cls.TAKE_PROFIT_PERCENTAGE}%")
        print(f"  TELEGRAM_ENABLED: {cls.TELEGRAM_ENABLED}")
        print(f"  LOG_LEVEL: {cls.LOG_LEVEL}")
        print(f"  PRIVATE_KEY: {'✅ Set' if cls.HYPERLIQUID_PRIVATE_KEY else '❌ Not Set'}")
        print(f"  SECRET_KEY: {'✅ Set' if cls.HYPERLIQUID_SECRET_KEY else '❌ Not Set'}")
        print(f"  TELEGRAM_BOT_TOKEN: {'✅ Set' if cls.TELEGRAM_BOT_TOKEN else '❌ Not Set'}")
        print(f"  TELEGRAM_CHAT_ID: {'✅ Set' if cls.TELEGRAM_CHAT_ID else '❌ Not Set'}") 